# File Name: autograder_core.py
# Author: Jonathan Morse

# Used to access command line arguments 
import sys
# Allows shell commands
import subprocess
from subprocess import Popen, PIPE
# Allows access to output system
import os

# Created to set up all the auto graders

    # Goes into the "Insert_Zip_output_Here" directory and records the
    # name of the zip file that contains each student's project. Unzips
    # the file and records the name of each file within. Then, unzips 
    # each student's file and returns a list of each student's WPI 
    # username and the name of the first zip file. 
def unzip_organize():
    
    zip_output_name = os.listdir("./Insert_Zip_File_Here")[0]
    # removes .zip from name
    output_name = zip_output_name.split('.')[0]
    # unzips file
    subprocess.run("cd ./Insert_Zip_File_Here; unzip %s" 
    	% (output_name),
                   shell=True,
                   stdout=subprocess.PIPE,
                   stderr=subprocess.PIPE
                   )

    list_student_zip_output = os.listdir("./Insert_Zip_File_Here/%s"
    	% (output_name))

    number_student_zip_outputs = len(list_student_zip_output)

    # Initializes list that will contain the name of each student's project
    list_student_outputs = []

    # Removes the ".zip" from every students file name 
    for x in range(0,number_student_zip_outputs):
        list_student_outputs.append(list_student_zip_output[x].split('.')[0])

    # Initializes list that will contain every students WPI user name
    list_students = []

    # Removes the "_PA<some number>" from every students file name 
    for x in range(0,number_student_zip_outputs):
        list_students.append(list_student_outputs[x].split('_')[0])

    # Unzips every students file and changes the folder name to just
    # the student's WPI username
    for x in range(0,number_student_zip_outputs):
        subprocess.run("cd ./Insert_Zip_File_Here/%s; unzip %s; mv %s %s" 
        	% (output_name, 
               list_student_outputs[x],
               list_student_outputs[x],
               list_students[x]),
                        shell=True,
                        stdout=subprocess.PIPE,
                        stderr=subprocess.PIPE
                        )

    return list_students, output_name

    # Runs provided command and checks if the return code is zero.
    # If return code is zero then function returns grade_possible
    # and a string saying the compile was successful. If return
    # code is not zero then the compile failed due to error or
    # warning, so the functions returns zero for the grade, and
    # that the compile failed for the reason_for_score. If the 
    # return code is zero that means the function was successful
def compile_test(command, grade_possible):

    compile_test_one = subprocess.run(command,
                                    shell=True,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)

    if compile_test_one.returncode == 0:
        grade_earned = grade_possible
        reason_for_score_for_score = "Compile was successful, no warnings or errors"
    else:
        grade_earned = 0         
        reason_for_score_for_score = "Compile failed, program had error(s) or warning(s)"

    return grade_earned, reason_for_score_for_score; 

    # Checks the grade earned from the compile test. If it
    # is not equal to the highest possible grade then the code
    # did not compile. So the function adds the student's name 
    # to the did_not_compile.txt output.
def compile_check(grade_earned, grade_possible, student_name):

    if grade_earned != grade_possible:
        compile_doc = open("../Output_Files/did_not_compile.txt", "w")
        compile_doc.write("\n%s" % (student_name))
    else:
        return

    return

    # Counts the number of files in the students folder then
    # runs "make clean", then recounts the number of files. 
    # If the initial count is greater then adds the students 
    # name to the not_clean.txt file. 
def clean_check(student_name):

    initial_dir_size = len(os.listdir("./%s" % (student_name)))
    
    clean_command = subprocess.run("make clean",
                                    shell=True,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)

    cleaned_dir_size = len(os.listdir("./%s" % (student_name)))

    if initial_dir_size > cleaned_dir_size:
        clean_doc = open("../Output_Files/not_clean.txt", "w")
        clean_doc.write("\n%s" % (student_name))
    else:
        return

    return



# First Autograder 

    # Creates a pipe and runs given command. Then inserts program_input
    # into the pipe and records stdout. Stdout is then decode from bytes
    # to a string. Then the string is turned into a list of strings, 
    # split at every new line character ("\n"). 
def test_case(command, program_input): 

    run_command = Popen(command, stdin=PIPE, stdout=PIPE)
 
    output_bytes = run_command.communicate(input=b'%d' % (program_input))[0]

    output_list = output_bytes.decode("utf8"). splitlines()

    return output_list; 


    # Runs the test_case function then checks to see if the two inputted
    # words are in any line of the output. Then returns the grade_earned 
    # and the reason_for_score for that grade. 
def search_word(command, 
                program_input, 
                word_one, 
                word_two, 
                grade_possible):

    output = test_case(command,
                       program_input)

    for x in range(0, len(output)):

        if word_one in output[x] and word_two in output[x]:
            grade_earned = grade_possible
            reason_for_score_for_score = "\n\tThe number '%d' was entered as user " \
                % (program_input) \
                               + "input, code successfully scanned user " \
                               + "input and printed it on screen, scanf()" \
                               + " implemented correctly"
            break
        else:
            grade_earned = 0 
            reason_for_score_for_score = "\n\tThe number '%d' was entered as user" \
                % (program_input) \
                               + " input, code unsuccessfully scanned user " \
                               + "input, scanf() was not implemented " \
                               + "correctly"
        
            continue

    return grade_earned, reason_for_score_for_score; 

    # Checks to ensure that a month starts on the proper day. It does
    # this by finding the line with specific month on it, then appending
    # the lines two and three lines down. The line two down should equal
    # "Sun Mon Tue Wed Thu Fri Sat" and the line three down should be
    # something like "  1  2  3  4  5  6  7". Then the functions checks 
    # if two specific characters match up. For example if the "t" in "Sat"
    # matches up with "7" on the line below.
def check_month(command, program_input, char1, 
                char2, month, grade_possible):

    output = test_case(command,program_input)

    # Initializes a list that will contain all strings of a particular month
    month_list = []

    for x in range(0,len(output)):

        if month in output[x]:
            month_list.append(output[x+2])
            month_list.append(output[x+3])
            break
        else:
            result = 0 
            continue

    # Error handling
    if char1 in month_list[0] and char2 in month_list[1]:

        if month_list[0].index(char1) == month_list[1].index(char2):
            grade_earned = grade_possible
            reason_for_score = "\n\tCorrectly printed first week of" \
                               + " calendar for year %d, printf()" \
                               % (program_input) \
                               + " implemented correctly"
        else:
            grade_earned = 0
            reason_for_score = "\n\tIncorrectly printed first week of" \
                               + " calendar for year %d, printf()" \
                               % (program_input) \
                               + " not implemented correctly"

    else:
        grade_earned = 0 
        reason_for_score = "\n\tCorrectly printed first week of" \
                           + " calendar for year %d, printf()" \
                           % (program_input) \
                           + " implemented correctly"

    return grade_earned, reason_for_score

    # Runs PA1_Test for user inputted year to get the correct
    # characters to insert into check_month, then runs the
    # check_month function for each month of the year. 
def check_year(command, program_input):
    # A list of strings from test program
    test_output = test_case("./PA1_Test",program_input)
    
    first_week = []

    result = 0

    # A list of all the months
    all_months = ["January",
                  "February",
                  "March",
                  "April",
                  "May",
                  "June",
                  "July",
                  "August",
                  "September",
                  "October",
                  "November",
                  "December"]

    for x in range(0,len(all_months)):

        for y in range(0,len(test_output)):

            if all_months[x] in test_output[y]:
                first_week.append(test_output[y+2])
                first_week.append(test_output[y+3])
                char_one = first_week[0][32]
                char_two = first_week[1][32]
                month_check = check_month(command, 
                                          program_input, 
                                          char_one, 
                                          char_two, 
                                          all_months[x],
                                          1)[0]
                result = month_check + result
                first_week.clear()

    if result == 12:
        final_result = 1
    else:
        final_result = 0 

    return final_result



# Created for programming assignment 2

    # This function test to see if each individual ".c" file
    # compiles, and also test to see if "make" and "make clean"
    # were implemented correctly
def make_all_files_test(grade_possible, student_name):
    # Creating a list of all file names in student's folder
    list_of_files = os.listdir("./%s" % (student_name))
    # Number of files in student's folder
    num_of_files = len(list_of_files)
    # Inializing a list that will contain the name of 
    # all ".c" files in folder
    code_files = []
    # Finding all the ".c" files
    for x in range(0,num_of_files):
        if ".c" in list_of_files[x]:
            # If a file ends in ".c" the ".c" is removed and
            # ".o" is added then the string is added to the code_files list
            code_files.append(list_of_files[x].split('.')[0] + ".o")
    # Number of source code files
    num_of_code_files = len(code_files)
    # Variables used to calculate if all compile test worked 
    total = 0
    # Set to 2 to account for "make" and "make_clean" tests
    tests_run = 2
    reason_for_score = ""
    # Goes into the folder and runs make 
    # for each individual ".o" file
    for x in range(0,num_of_code_files):
        output_make_test = compile_test("cd ./%s; make %s" 
            % (student_name, code_files[x]), 1)
        total = total + output_make_test[0]
        reason_for_score = reason_for_score \
                 + "\n\t" \
                 + output_make_test[1] \
                 + " when running command: make %s" % (code_files[x])
        tests_run = tests_run + 1
    # Runs make on the entire project
    make_test = compile_test("cd ./%s; make" % (student_name), 1)
    total = total + make_test[0]
    reason_for_score = reason_for_score \
                       + "\n\t" \
                       + make_test[1] \
                       + " when running command: make "
    # Test to make sure "make clean" works correctly
    make_clean_test = compile_test("cd ./%s; make clean" 
        % (student_name), 1)
    total = total + make_clean_test[0]
    reason_for_score = reason_for_score \
             + "\n\t" \
             + make_clean_test[1] \
             + " when running command: make clean"

    if total == tests_run:
        grade_earned = grade_possible
        return grade_earned, reason_for_score
    else:
        grade_earned = 0
        return grade_earned, reason_for_score

    # Runs a command that does not need inputs when running,
    # unlike test_case and returns stdout as a list of strings. 
def test_case_two(command):

    output = subprocess.run(command,
                                  shell=True,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)

    output_list = output.stdout.decode("utf8").splitlines()

    return output_list

    # Test to ensure that the expected array that is created
    # when running Programming Assignment Two is what is 
    # actually created. Does this by reading a file that 
    # contains the correct array and comparing that to the
    # array created by the student
def array_compare(command, answer_output, num_of_lines): 

    student_output = test_case_two(command)
    
    # Turn a output into a list line by line 
    output = open(answer_output,"r")
    read_output = output.read().splitlines()

    counter = 0

    for x in range(0,num_of_lines):
        if read_output[x] == student_output[x]:
            counter = counter + 1
        else:
            continue 

    if counter == num_of_lines:
        result = 1
    else:
        result = 0 

    return result

    # This function first ensures that the proper array is produced
    # using array compare, then ensures the program stopped due to 
    # the correct termination condition. 
def termination_condition(command, answer_output, num_of_lines, condition):
    # Tests to see if proper array is produced
    counter = array_compare(command, answer_output, num_of_lines)
    # Records entire output of command
    command_output = test_case_two(command)

    for x in range(0,len(command_output)):
        if condition in command_output[x]:
            counter = counter + 1
        else:
            continue 

    if counter == 2:
        result = 1
    else:
        result = 0 
    
    return result 

# Created for programming assignment 3

    # Counts the number of ".c" files in a folder. Then
    # returns a grade and a reason for that grade based 
    # on the number of ".c" files. 
def counting_modules(grade_possible,student_name):
    # Creating a list with all files in student's folder
    list_of_files = os.listdir("./%s" % (student_name))
    
    num_of_files = len(list_of_files)
    # Creating a list of all ".c" files
    list_of_c_files = []
    # Creating output counter
    num_of_c_files = 0
    # Finding all the ".c" outputs
    for x in range(0,num_of_files):
        if ".c" in list_of_files[x]:
            num_of_c_files = num_of_c_files + 1
        else:
            continue

    if num_of_c_files >= 3:
        grade_earned = grade_possible
        reason_for_score = "Project has three or more modules"
    else:
        grade_earned = 0 
        reason_for_score = "Project has less than three modules"

    return grade_earned, reason_for_score

    # Runs a command, and then scans the output file that is 
    # produced. It then divides the output file at the line that
    # starts with "---" which in Programming Assignment Three 
    # divides the individual word section from the total word section.
    # Lastly it returns three lists of strings, the first is all the 
    # lines in the output file, the second is the individual word section
    # of the output file and the last is the total word section of the 
    # output files. 
def word_scan(command, output_name):
    command_output = test_case_two(command)
    output = open(output_name,"r")
    word_output = output.read().splitlines()
    total_word_output = []
    individual_word_output = []

    for x in range(0,len(word_output)):
        if "---" in word_output[x]:
            total_word_output.append(word_output[x+1])
            total_word_output.append(word_output[x+2])
            for y in range(0,(x-1)):
                individual_word_output.append(word_output[y])
        else:
            continue

    return word_output, total_word_output, individual_word_output

    # Checks to see if the command properly counts the
    # total number of unique words and total number 
    # of words overall in the input files(s)
def total_word_scan(command, output_name, test_name, grade_possible, comment):
    total_word_output = word_scan(command, output_name)[1]
    test_output = open(test_name,"r")
    test_output_list = test_output.read().splitlines()

    if test_output_list[0] in total_word_output[0] \
        and test_output_list[1] in total_word_output[1]: 

        grade_earned = grade_possible
        reason_for_score_for_score = comment + " was successful"
    else:
        grade_earned = 0
        reason_for_score_for_score = comment + " was unsuccessful"

    return grade_earned, reason_for_score_for_score

    # Checks to see if the command properly counts the total 
    # number of each individual word in the input files(s)
def individual_word_scan(command, output_name, test_name,
                         grade_possible, comment):
    individual_word_output = word_scan(command, output_name)[2]
    test_output = open(test_name,"r")
    test_output_output = test_output.read().splitlines()

    total_counter = 0
    correct_counter = 0 

    for x in range(0,len(individual_word_output)):
        total_counter = total_counter + 1 
        if test_output_output[x] == total_word_output[x]: 
            correct_counter = correct_counter + 1
        else:
            continue

    if total_counter == correct_counter:
        grade_earned = grade_possible
        reason_for_score_for_score = comment + " was successful"
    else:
        grade_earned = 0
        reason_for_score_for_score = comment + " was unsuccessful"

    return grade_earned, reason_for_score_for_score

# Created to make grading rubric     

    # Creates the header for a rubric text file. 
def header(assignment_number, student_name, grader_first_name,
           grader_last_name, point_total):

    # Centers the title in a space of 100 total characters 
    title = "PROGRAMMING ASSIGNMENT %s RUBRIC" \
        % (assignment_number)

    # WPI username of student completing assignment 
    student_name = "Student WPI username: %s" \
        % (student_name)

    # Full name of grader
    grader_name = "Assignment graded by: %s %s" \
        % (grader_first_name,
           grader_last_name)

    # Total points for the programming assignment 
    point_statement = "This programming assignment is worth: %s points" \
        % (point_total)

    # Formatting
    header = "%s\n\n%s\n\n%s\n\n%s" \
        % (title, 
           student_name, 
           grader_name, 
           point_statement)

    return header; 

    # Creates a task description for the rubric text file. 
def task_create(task_number, task_description, points_possible, 
               points_earned, reason_for_score):

    task_statement = "Task %s: %s" \
        % (task_number,
           task_description)

    points_possible_statement = "Maximum Points: %d" \
        % (points_possible)

    points_earned_statement = "Earned Points: %d" \
        % (points_earned)

    reason_for_score_statement = "reason for score: %s" \
        % (reason_for_score)

    # Formatting
    task = "\n\n%s\n\n\t%s\n\n\t%s\n\n\t%s" \
        %  (task_statement, 
            points_possible_statement, 
            points_earned_statement, 
            reason_for_score_statement)

    return task;








    

    

